@@ -108,7 +108,7 @@ void log_ident_set(const char* id);
  * @param t: the point is copied and used to find the time.
  * 	if NULL, time(2) is used.
  */
-void log_set_time(uint32_t* t);
+void log_set_time(time_t* t);
 
 /**
  * Set if the time value is printed ascii or decimal in log entries.
