@@ -41,6 +41,10 @@
  */
 
 #include "config.h"
+#ifdef HAVE_TIME_H
+#include <time.h>
+#endif
+#include <sys/time.h>
 
 #if defined(USE_MINI_EVENT) && !defined(USE_WINSOCK)
 #include <signal.h>
@@ -75,13 +79,13 @@ settime(struct event_base* base)
 		return -1;
 	}
 #ifndef S_SPLINT_S
-	*base->time_secs = (uint32_t)base->time_tv->tv_sec;
+	*base->time_secs = (time_t)base->time_tv->tv_sec;
 #endif
 	return 0;
 }
 
 /** create event base */
-void *event_init(uint32_t* time_secs, struct timeval* time_tv)
+void *event_init(time_t* time_secs, struct timeval* time_tv)
 {
 	struct event_base* base = (struct event_base*)malloc(
 		sizeof(struct event_base));
