@@ -46,7 +46,8 @@
 #include "openssl/ssl.h"
 #endif
 #ifdef HAVE_NSS
-#include <nss3/sechash.h>
+/* nss3 */
+#include "sechash.h"
 #endif
 #include "validator/val_nsec3.h"
 #include "validator/validator.h"
@@ -712,6 +713,8 @@ nsec3_hash_name(rbtree_t* table, struct regional* region, ldns_buffer* buf,
 		return r;
 #ifdef UNBOUND_DEBUG
 	n =
+#else
+	(void)
 #endif
 	rbtree_insert(table, &c->node);
 	log_assert(n); /* cannot be duplicate, just did lookup */
